#include <poll.h>
#include <stdint.h>
#include <stdlib.h>

#if defined(__linux__)
#include <bsd/stdlib.h>
#endif

#include "stun.h"

int
challenge(int netfd)
{
	unsigned char buf[sizeof(uint64_t)];
	size_t outlen;
	struct pollfd pfd[1];
	uint64_t n, reply;
	int ret;

	arc4random_buf(&n, sizeof(buf));
	pack64(buf, n);
	if (netwrite(netfd, buf, sizeof(buf), &outlen) == PKTFAILED)
		return -1;

	pfd[0].fd = netfd;
	pfd[0].events = POLLIN;
	for (;;) {
		ret = poll(pfd, 1, AUTHTIMEO * 1000);
		if (ret < 0) {
			logwarn("poll failed");
			return -1;
		} else if (ret == 0) {
			logwarn("challenge-response timed out");
			return -1;
		}

		if (pfd[0].revents & (POLLIN | POLLHUP)) {
			ret = netread(netfd, buf, sizeof(buf), &outlen);
			if (ret == PKTFAILED)
				return -1;
			else if (ret == PKTPARTIAL)
				continue;
			if (outlen != sizeof(buf))
				return -1;
			reply = unpack64(buf);
			if (n + 1 == reply)
				return 0;
		}
	}
	return -1;
}

int
response(int netfd)
{
	unsigned char buf[sizeof(uint64_t)];
	size_t outlen;
	struct pollfd pfd[1];
	uint64_t reply;
	int ret;

	pfd[0].fd = netfd;
	pfd[0].events = POLLIN;
	for (;;) {
		ret = poll(pfd, 1, AUTHTIMEO * 1000);
		if (ret < 0) {
			logwarn("poll failed");
			return -1;
		} else if (ret == 0) {
			logwarn("challenge-response timed out");
			return -1;
		}

		if (pfd[0].revents & (POLLIN | POLLHUP)) {
			ret = netread(netfd, buf, sizeof(buf), &outlen);
			if (ret == PKTFAILED)
				return -1;
			else if (ret == PKTPARTIAL)
				continue;
			if (outlen != sizeof(buf))
				return -1;
			reply = unpack64(buf);
			pack64(buf, reply + 1);
			if (netwrite(netfd, buf, sizeof(buf), &outlen) == PKTFAILED)
				return -1;
			break;
		}
	}
	return 0;
}
