#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <string.h>
#include <unistd.h>

#include "stun.h"

int
clientconnect(char *host, char *port)
{
	struct addrinfo hints, *ai, *p;
	int ret, netfd;

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	if ((ret = getaddrinfo(host, port, &hints, &ai)))
		logerr("getaddrinfo: %s", gai_strerror(ret));

	for (p = ai; p; p = p->ai_next) {
		if (p->ai_family != aftype)
			continue;
		if ((netfd = socket(p->ai_family, p->ai_socktype,
		                    p->ai_protocol)) < 0)
			continue;
		if (connect(netfd, p->ai_addr, p->ai_addrlen) < 0) {
			close(netfd);
			continue;
		}
		break;
	}
	freeaddrinfo(ai);
	if (!p) {
		logwarn("failed to connect to %s:%s", host, port);
		return -1;
	}

	setnonblock(netfd, 1);
	setsockopt(netfd, SOL_SOCKET, SO_KEEPALIVE, (int []){1}, sizeof(int));
	setsockopt(netfd, IPPROTO_TCP, TCP_NODELAY, (int []){1}, sizeof(int));

	if (response(netfd) < 0 || challenge(netfd) < 0) {
		close(netfd);
		logwarn("challenge-response failed");
		return -1;
	}
	return netfd;
}
