#include <sys/types.h>
#include <sys/uio.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <net/if.h>
#if defined(__OpenBSD__) || defined(__FreeBSD__)
#include <net/if_tun.h>
#elif defined(__DragonFly__)
#include <net/tun/if_tun.h>
#endif
#include <netinet/in.h>

#include <fcntl.h>
#include <libgen.h>
#include <limits.h>
#include <stdio.h>
#include <unistd.h>

#include "stun.h"

int
devopen(char *ifname)
{
	char dev[PATH_MAX];
	struct tuninfo ti;
	int fd;

	snprintf(dev, sizeof(dev), "/dev/%s", basename(ifname));
	if ((fd = open(dev, O_RDWR)) < 0)
		logerr("failed to open %s", dev);
	if (ioctl(fd, TUNGIFINFO, &ti) < 0)
		logerr("failed to set TUNGIFINFO on %s", dev);
	if (devtype == TUNDEV)
		ti.mtu = MAXPAYLOADLEN;
	else
		ti.mtu = MAXPAYLOADLEN - 14; /* make some room for ethernet header */
	if (ioctl(fd, TUNSIFINFO, &ti) < 0)
		logerr("failed to set TUNSIFINFO on %s", dev);
	if (devtype == TUNDEV) {
#if defined(TUNSIFHEAD)
		int one = 1;
		if (ioctl(fd, TUNSIFHEAD, &one) < 0)
			logerr("failed to set TUNSIFHEAD on %s", dev);
#endif
	}
	return fd;
}

int
devwrite(int fd, unsigned char *buf, int len)
{
	struct iovec iov[2];
	uint32_t type;
	int n, af;

	switch (devtype) {
	case TAPDEV:
		return write(fd, buf, len);
	case TUNDEV:
		if ((af = ipversion(buf)) < 0)
			return -1;
		type = htonl(af);
		iov[0].iov_base = &type;
		iov[0].iov_len = sizeof(type);
		iov[1].iov_base = buf;
		iov[1].iov_len = len;
		n = writev(fd, iov, 2);
		if (n > 0)
			n -= sizeof(type);
		break;
	}
	return n;
}

int
devread(int fd, unsigned char *buf, int len)
{
	struct iovec iov[2];
	uint32_t type;
	int n;

	switch (devtype) {
	case TAPDEV:
		return read(fd, buf, len);
	case TUNDEV:
		iov[0].iov_base = &type;
		iov[0].iov_len = sizeof(type);
		iov[1].iov_base = buf;
		iov[1].iov_len = len;
		n = readv(fd, iov, 2);
		if (n > 0)
			n -= sizeof(type);
		break;
	}
	return n;
}
