#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <net/if.h>
#include <linux/if_tun.h>

#include <fcntl.h>
#include <string.h>
#include <unistd.h>

#include "stun.h"

int
devopen(char *ifname)
{
	struct ifreq ifr;
	int fd, s;

	if ((fd = open("/dev/net/tun", O_RDWR)) < 0)
		logerr("failed to open %s", "/dev/net/tun");

	memset(&ifr, 0, sizeof(ifr));
	ifr.ifr_flags = (devtype == TUNDEV ? IFF_TUN : IFF_TAP) | IFF_NO_PI;
	strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
	ifr.ifr_name[IF_NAMESIZE - 1] = '\0';
	if (ioctl(fd, TUNSETIFF, &ifr) < 0)
		logerr("failed to set TUNSETIFF on %s", ifname);

	/* dummy socket so we can manipulate the params */
	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		logerr("failed to create socket");
	if (devtype == TUNDEV)
		ifr.ifr_mtu = MAXPAYLOADLEN;
	else
		ifr.ifr_mtu = MAXPAYLOADLEN - 14; /* make some room for ethernet header */
	if (ioctl(s, SIOCSIFMTU, &ifr) < 0)
		logerr("failed to set MTU on %s", ifname);
	close(s);

	return fd;
}

int
devwrite(int fd, unsigned char *buf, int len)
{
	return write(fd, buf, len);
}

int
devread(int fd, unsigned char *buf, int len)
{
	return read(fd, buf, len);
}
