#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>

#include "stun.h"

static char *progname;

static void
logmsg(int priority, char *msg, va_list ap)
{
	if (debug) {
		fprintf(stderr, "%s: ", progname);
		vfprintf(stderr, msg, ap);
		fputc('\n', stderr);
	} else {
		vsyslog(priority, msg, ap);
	}
}

void
loginit(char *prog)
{
	progname = prog;
	if (!debug)
		openlog(progname, LOG_PID | LOG_NDELAY, LOG_DAEMON);
}

void
logdbg(char *msg, ...)
{
	va_list ap;

	va_start(ap, msg);
	logmsg(LOG_DAEMON | LOG_DEBUG, msg, ap);
	va_end(ap);
}

void
logwarn(char *msg, ...)
{
	va_list ap;

	va_start(ap, msg);
	logmsg(LOG_DAEMON | LOG_WARNING, msg, ap);
	va_end(ap);
}

void
logerr(char *msg, ...)
{
	va_list ap;

	va_start(ap, msg);
	logmsg(LOG_DAEMON | LOG_ERR, msg, ap);
	va_end(ap);
	exit(1);
}
