#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <errno.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>

#include "stun.h"

int
serverinit(char *host, char *port)
{
	struct addrinfo hints, *ai, *p;
	int ret, listenfd;

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_flags = AI_PASSIVE;

	if ((ret = getaddrinfo(host, port, &hints, &ai)))
		logerr("getaddrinfo: %s", gai_strerror(ret));

	for (p = ai; p; p = p->ai_next) {
		if (p->ai_family != aftype)
			continue;
		if ((listenfd = socket(p->ai_family, p->ai_socktype,
		                       p->ai_protocol)) < 0)
			continue;
		setsockopt(listenfd, SOL_SOCKET, SO_REUSEADDR, (int []){1},
		           sizeof(int));
		if (bind(listenfd, p->ai_addr, p->ai_addrlen) < 0) {
			close(listenfd);
			continue;
		}
		if (listen(listenfd, 5) < 0) {
			close(listenfd);
			continue;
		}
		break;
	}
	if (!p)
		logerr("failed to bind socket");
	freeaddrinfo(ai);
	return listenfd;
}

int
serveraccept(int listenfd)
{
	struct sockaddr_storage remote;
	int netfd;

	netfd = accept(listenfd, (struct sockaddr *)&remote,
	               (socklen_t []){sizeof(remote)});
	if (netfd < 0) {
		if (errno != ECONNABORTED)
			logwarn("accept failed");
		return -1;
	}

	setnonblock(netfd, 1);
	setsockopt(netfd, SOL_SOCKET, SO_KEEPALIVE, (int []){1}, sizeof(int));
	setsockopt(netfd, IPPROTO_TCP, TCP_NODELAY, (int []){1}, sizeof(int));

	if (challenge(netfd) < 0 || response(netfd) < 0) {
		close(netfd);
		logwarn("challenge-response failed");
		return -1;
	}
	return netfd;
}
