#include <sys/socket.h>

#include <stddef.h>
#include <stdint.h>

#define NOPRIVUSER      "nobody"
#define AUTHTIMEO       10 /* in seconds */
#define RECONNECTTIMEO  60 /* in seconds */
#define HDRLEN          2
#define MAXPAYLOADLEN   1404
#define NROUNDS         100000
#define DEFPORT         "12080"
#define DEFCIPHER       "chacha20-poly1305"

enum {
	PKTFAILED,
	PKTPARTIAL,
	PKTCOMPLETE
};

enum {
	TUNDEV,
	TAPDEV
};

/* stun.c */
extern int devtype;
extern int aftype;
extern int debug;

/* auth.c */
int challenge(int);
int response(int);

/* client.c */
int clientconnect(char *, char *);

/* crypto.c */
void listciphers(void);
void setcipher(char *);
void derivekey(char *);
void cryptoinit(void);
size_t cryptononcelen(void);
size_t cryptotaglen(void);
int cryptoseal(unsigned char *, size_t *, size_t, const unsigned char *,
               size_t, const unsigned char *, size_t, const unsigned char *,
               size_t);
int cryptoopen(unsigned char *, size_t *, size_t, const unsigned char *,
               size_t, const unsigned char *, size_t, const unsigned char *,
               size_t);

/* dev_*.c */
int devopen(char *);
int devwrite(int, unsigned char *, int);
int devread(int, unsigned char *, int);

/* log.c */
void loginit(char *);
void logdbg(char *, ...);
void logwarn(char *, ...);
void logerr(char *, ...);

/* netpkt.c */
int netwrite(int, unsigned char *, size_t, size_t *);
int netread(int, unsigned char *, size_t, size_t *);
void netreset(void);
void netinit(void);

/* server.c */
int serverinit(char *, char *);
int serveraccept(int);

/* tunnel.c */
int tunnel(int, int);

/* util.c */
void pack16(unsigned char *, uint16_t);
uint16_t unpack16(unsigned char *);
void pack64(unsigned char *, uint64_t);
uint64_t unpack64(unsigned char *);
void revokeprivs(void);
int setnonblock(int, int);
char *saddr_ntop(struct sockaddr *, socklen_t);
char *peer_ntop(int);
int ipversion(unsigned char *);
