#include <sys/types.h>

#include <netinet/in.h>
#include <netinet/ip.h>
#include <netdb.h>

#include <fcntl.h>
#include <grp.h>
#include <pwd.h>
#include <stdint.h>
#include <stdio.h>
#include <unistd.h>

#include "stun.h"

void
pack16(unsigned char *buf, uint16_t n)
{
	buf[0] = n >> 8 & 0xff;
	buf[1] = n & 0xff;
}

uint16_t
unpack16(unsigned char *buf)
{
	return buf[0] << 8 | buf[1];
}

void
pack64(unsigned char *buf, uint64_t n)
{
	buf[0] = n >> 56 & 0xff;
	buf[1] = n >> 48 & 0xff;
	buf[2] = n >> 40 & 0xff;
	buf[3] = n >> 32 & 0xff;
	buf[4] = n >> 24 & 0xff;
	buf[5] = n >> 16 & 0xff;
	buf[6] = n >> 8 & 0xff;
	buf[7] = n & 0xff;
}

uint64_t
unpack64(unsigned char *buf)
{
	return (uint64_t)buf[0] << 56 |
	       (uint64_t)buf[1] << 48 |
	       (uint64_t)buf[2] << 40 |
	       (uint64_t)buf[3] << 32 |
	       (uint64_t)buf[4] << 24 |
	       (uint64_t)buf[5] << 16 |
	       (uint64_t)buf[6] << 8  |
	       (uint64_t)buf[7];
}

void
revokeprivs(void)
{
	struct passwd *pw;

	if (!(pw = getpwnam(NOPRIVUSER)))
		logerr("no %s user", NOPRIVUSER);
	if (setgroups(1, &pw->pw_gid) < 0 ||
	    setresgid(pw->pw_gid, pw->pw_gid, pw->pw_gid) < 0 ||
	    setresuid(pw->pw_uid, pw->pw_uid, pw->pw_uid) < 0)
		logerr("failed to revoke privs");
}

int
setnonblock(int fd, int mode)
{
	int flags;

	flags = fcntl(fd, F_GETFL);
	if (mode)
		flags |= O_NONBLOCK;
	else
		flags &= ~O_NONBLOCK;
	return fcntl(fd, F_SETFL, flags);
}

char *
saddr_ntop(struct sockaddr *sa, socklen_t salen)
{
	static char buf[512];
	char host[NI_MAXHOST], port[NI_MAXSERV];
	int ret;

	if ((ret = getnameinfo(sa, salen, host, sizeof(host),
	                       port, sizeof(port),
	                       NI_NUMERICHOST | NI_NUMERICSERV))) {
		if (ret == EAI_SYSTEM) {
			logwarn("getnameinfo failed");
			return NULL;
		} else {
			logwarn("getnameinfo: %s", gai_strerror(ret));
			return NULL;
		}
	}
	snprintf(buf, sizeof(buf), "[%s]:%s", host, port);
	return buf;
}

char *
peer_ntop(int fd)
{
	struct sockaddr_storage ss;
	socklen_t sslen = sizeof(ss);

	if (getpeername(fd, (struct sockaddr *)&ss, &sslen) < 0) {
		logwarn("getpeername failed");
		return NULL;
	}
	return saddr_ntop((struct sockaddr *)&ss, sslen);
}

int
ipversion(unsigned char *pkt)
{
	struct ip *ip = (struct ip *)pkt;

	switch (ip->ip_v) {
	case 4:
		return AF_INET;
	case 6:
		return AF_INET6;
	default:
		logwarn("unknown protocol version: %d", (int)ip->ip_v);
		break;
	}
	return -1;
}
